(** Aide à la détection de bugs des L-trominos *)

(** VOUS NE TROUVEREZ RIEN ICI QUI SOIT UN CORRIGE OU QUI VOUS AIDE À INVENTER LE CODE DEMANDÉ 
    N'ESSAYEZ PAS DE COMPRENDRE CE FICHIER.
*)


(** Une case est la paire (i, j) de ses coordonnées *)
type case = int*int

(** Un tromino est un triplet de cases *)
type tromino = case * case * case

(** Un pavage est une liste de trominos *)
type pavage = tromino list

let sort_paires =
  let aux (a,b) (c,d) =
    if a = c then b-a
    else a-c
  in
  List.stable_sort aux


let est_piece ((alp : case), (bet : case), (gam : case)) =
  let [@warning "-8"]  [(a,b); (c,d); (e,f)] = sort_paires [alp; bet; gam] in
  (*  (a,b) (c,d)
      (e,f)
  *)
  e = a+1 && b = f && c = a && d = b+1
  ||
  (* (a,b)
     (c,d) (e,f) 
     
  *)
  c = a+1 && d = b && e = c && f = d+1
  ||
  (*       (a,b)
     (c,d) (e,f)
  *)
  c = e && f = d+1 && e = a+1 && b = f
  ||
  (*
    (a,b) (c,d)
          (e,f) 
  *)
  c = a && d = b+1 && e = a+1 && d = f


exception Break

let check ((orig_x, orig_y) : case) (dim : int) ((abs_x, abs_y) : case) (pieces : pavage) : bool =

  (* Crée le tableau qui stocke quelles cases sont recouvertes *)
  let t = Array.make_matrix dim dim false in 
  t.(abs_x - orig_x).(abs_y - orig_y) <- true;

  (* Fonction auxiliaire qui essaye de mettre une case à true *)
  let update_one_coo (abs, ord) =
    if t.(abs - orig_x).(ord - orig_y) then begin
      Printf.printf "La case (%d,%d) a été couverte deux fois.\n" abs ord;
      false
    end else begin
      t.(abs - orig_x).(ord - orig_y) <- true;
      true
    end
  in

  (* Parcourt les pièces et les place au fur et à mesure. *)
  let rec loop = function 
    | [] -> true
    | p :: _ when not (est_piece p) -> 
        begin
          let ( (a,b), (c,d), (e,f) ) = p in
            Printf.printf "( (%d,%d), (%d,%d), (%d,%d) ) n'est pas une pièce.\n" a b c d e f;
            false
        end 
    | ( (a,b), (c,d), (e,f) ) :: q ->
        begin
          if update_one_coo (a,b) 
             && update_one_coo (c,d) 
             && update_one_coo (e,f) then loop q
          else false
        end
  in

  (* On parcourt toutes les pièces, 
     puis vérifie que toutes les cases sont recouvertes 
  *)
  if not (loop pieces) then 
    (* On a déjà eu un message d'erreur avant. *)
    false
  else
    (* Vérifier que toutes les cases sont recouvertes. *)
    let vide = ref (-1,-1) in
    try begin
      for i = 0 to dim-1 do
        for j = 0 to dim-1 do
          if not (t.(i).(j)) then begin
            vide := (i,j);
            raise Break
          end
        done
      done;
      Printf.printf "Ce pavage est valide. Bravo !\n";
      true
    end
    with Break -> begin
      let (i,j) = !vide in
      Printf.printf "La case (%d, %d) n'est pas couverte." (i + orig_x) (j + orig_y);
      false
    end



let print_cases_couvertes ((orig_x, orig_y) : case) (dim : int) ((abs_x, abs_y) : case) (pieces : pavage) =
  let t = Array.make_matrix dim dim 0 in 
  t.(abs_x - orig_x).(abs_y - orig_y) <- 1;

  let aux ((a,b), (c,d), (e,f)) =
    t.(a).(b) <- t.(a).(b) +1;
    t.(c).(d) <- t.(c).(d) +1;
    t.(e).(f) <- t.(e).(f) +1
  in
  List.iter aux pieces;

  for i = 0 to dim -1 do
    for j = 0 to dim-1 do
      Printf.printf "%d " t.(i).(j)
    done;
    print_newline ()
  done

